#include "BMP180.h"
#include "math.h"

/*
 * Starts the I2C peripheral and waits until BMP180 initializes
 * @param: 
 *          none
 * @returns: none
 */
void BMP180Init(void){
    I2C_Start(); 
    CyDelay(10);        // wait 10 ms
}

/*
 * The functions reads the calibration data from BMP180's EEPROM and creates a bit field stucture
 * @param: 
 *          none
 * @returns: BMP180_CALIBRATION_DATA
 */
BMP180_CALIBRATION_DATA makeBMPCalibrationTable(void){
    BMP180_CALIBRATION_DATA BMP180_calibration_data; 
    static uint8 I2CWriteBuffer[0x01] = {0xAA};         // Start of calibration data
    uint8 SensorCalibrationData[22];                  // Calibration data 
        
    /* Do a write operation with the memory address bytes to get calibration data */
	I2C_I2CMasterWriteBuf(BMP180_MODULE_ADDRESS, (uint8 *) I2CWriteBuffer, sizeof(I2CWriteBuffer), I2C_I2C_MODE_COMPLETE_XFER);
    while(!(I2C_I2CMasterStatus() & I2C_I2C_MSTAT_WR_CMPLT)); //Wait till the master completes writing
    I2C_I2CMasterClearStatus(); //Clear I2C master status
    
    /* Read calibration from the sensor EEPROM and store it in a buffer */
	I2C_I2CMasterReadBuf(BMP180_MODULE_ADDRESS, (uint8 *) SensorCalibrationData, sizeof(SensorCalibrationData), I2C_I2C_MODE_COMPLETE_XFER);
    while(!(I2C_I2CMasterStatus() & I2C_I2C_MSTAT_RD_CMPLT)); //Wait till the master completes reading
    I2C_I2CMasterClearStatus(); //Clear I2C master status
     
    BMP180_calibration_data = makeCalibrationData(SensorCalibrationData);
    
    return BMP180_calibration_data;
}

/*
 * The function assigns calibraton values to BMP180_CALIBRATION_DATA structure
 * @param: 
 *          uint8 *dBuffer - pointer to the buffer that holds the calibration data for the 
 *                      BMP180 sensor
 * @returns: BMP180_CALIBRATION_DATA struct
 */
BMP180_CALIBRATION_DATA makeCalibrationData(uint8 * dBuffer){
   BMP180_CALIBRATION_DATA data; 
    

    data.AC1 = (dBuffer[0] << 0x08) | dBuffer[1];
    data.AC2 = (dBuffer[2] << 0x08) | dBuffer[3];
    data.AC3 = (dBuffer[4] << 0x08) | dBuffer[5];
    data.AC4 = (dBuffer[6] << 0x08) | dBuffer[7];
    data.AC5 = (dBuffer[8] << 0x08) | dBuffer[9];
    data.AC6 = (dBuffer[10] << 0x08) | dBuffer[11];
    data.B1 = (dBuffer[12] << 0x08) | dBuffer[13];
    data.B2 = (dBuffer[14] << 0x08) | dBuffer[15];
    data.MB = (dBuffer[16] << 0x08) | dBuffer[17];
    data.MC = (dBuffer[18] << 0x08) | dBuffer[19];
    data.MD = (dBuffer[20] << 0x08) | dBuffer[21];
    
    uint8 breakline = 0;
    
    return data;    
}
/*
 * The function reads uncompensated temperature value from BMP180 sensor
 * @param: 
 *          none
 * @returns: uncompensated temperature value
 */
long BMP180_Read_UT(void) {
    long ut; 
    static uint8 I2CReadBuffer[0x02]; //The uncompensated temperature read from the sensor is stored here
    static uint8 I2CWriteBuffer[0x02] = {0xF4, 0x2E}; //Command for temperature conversion - Register address and value
    static uint8 SetReadAddressBuffer[0x01] = {0xF6}; //Register address from which the data is read
    
    /* Do a write operation with the memory address bytes to get temperature data */
	I2C_I2CMasterWriteBuf(BMP180_MODULE_ADDRESS, (uint8 *) I2CWriteBuffer, sizeof(I2CWriteBuffer), I2C_I2C_MODE_COMPLETE_XFER);
    while(!(I2C_I2CMasterStatus() & I2C_I2C_MSTAT_WR_CMPLT)); //Wait till the master completes writing
    I2C_I2CMasterClearStatus(); //Clear I2C master status
    
    CyDelay(5); //Wait for 5ms temperature conversion
    
    /* In order to do a read from a specific address, do a write operation with the memory address bytes alone */
	I2C_I2CMasterWriteBuf(BMP180_MODULE_ADDRESS, (uint8 *) SetReadAddressBuffer, sizeof(SetReadAddressBuffer), I2C_I2C_MODE_COMPLETE_XFER);
    while(!(I2C_I2CMasterStatus() & I2C_I2C_MSTAT_WR_CMPLT)); //Wait till the master completes writing
    I2C_I2CMasterClearStatus(); //Clear I2C master status
    
    /* Read temperature from the sensor and store it in a buffer */
	I2C_I2CMasterReadBuf(BMP180_MODULE_ADDRESS, (uint8 *) I2CReadBuffer, sizeof(I2CReadBuffer), I2C_I2C_MODE_COMPLETE_XFER);
    while(!(I2C_I2CMasterStatus() & I2C_I2C_MSTAT_RD_CMPLT)); //Wait till the master completes reading
    I2C_I2CMasterClearStatus(); //Clear I2C master status
    
	ut = (I2CReadBuffer[0x00] << 0x08) | I2CReadBuffer[0x01];     
    return ut;
}

/*
 * The function reads uncompensated preassure value from BMP180 sensor
 * @param: 
 *          none
 * @returns: uncompensated preassure value
 */
long BMP180_Read_UP(void) {
    long up;
    static uint8 I2CReadBuffer[0x03]; //The uncompensated pressure read from the sensor is stored here
    static uint8 I2CWriteBuffer[0x02] = {0xF4, 0x34 + (OSS << 0x06)}; //Command for pressure conversion - Register address and value
    static uint8 SetReadAddressBuffer[0x01] = {0xF6}; //Register address from which the data is read
    
    /* Do a write operation with the memory address bytes to get pressure data */
	I2C_I2CMasterWriteBuf(BMP180_MODULE_ADDRESS, (uint8 *) I2CWriteBuffer, sizeof(I2CWriteBuffer), I2C_I2C_MODE_COMPLETE_XFER);
    while(!(I2C_I2CMasterStatus() & I2C_I2C_MSTAT_WR_CMPLT)); //Wait till the master completes writing
    I2C_I2CMasterClearStatus(); //Clear I2C master status
    
    CyDelay(26); //Wait for 26ms pressure conversion
    
    /* In order to do a read from a specific address, do a write operation with the memory address bytes alone */
	I2C_I2CMasterWriteBuf(BMP180_MODULE_ADDRESS, (uint8 *) SetReadAddressBuffer, sizeof(SetReadAddressBuffer), I2C_I2C_MODE_COMPLETE_XFER);
    while(!(I2C_I2CMasterStatus() & I2C_I2C_MSTAT_WR_CMPLT)); //Wait till the master completes writing
    I2C_I2CMasterClearStatus(); //Clear I2C master status
    
    /* Read pressure from the sensor and store it in a buffer */
	I2C_I2CMasterReadBuf(BMP180_MODULE_ADDRESS, (uint8 *) I2CReadBuffer, sizeof(I2CReadBuffer), I2C_I2C_MODE_COMPLETE_XFER);
    while(!(I2C_I2CMasterStatus() & I2C_I2C_MSTAT_RD_CMPLT)); //Wait till the master completes reading
    I2C_I2CMasterClearStatus(); //Clear I2C master status
    
	up = ((I2CReadBuffer[0x00] << 0x10) | (I2CReadBuffer[0x01] << 0x08) | I2CReadBuffer[0x02]) >> (0x08 - OSS); 
    return up;
}

/* 
 * The function calculates the true temperature and returns in degrees Celsius. 
 * @param:
 *          long UT - uncompensated temperature value
 *          BMP180_CALIBRATION_DATA calibration - calibration data
 * @returns: true temperature value 
 */
long calculateTrueTemp(long UT, BMP180_CALIBRATION_DATA calibration){
    long Temperature; 
     long X1, X2, B5; 
    
    X1 = (UT - calibration.AC6) * calibration.AC5 / pow(2, 15);
    X2 = calibration.MC * pow(2, 11) / (X1 + calibration.MD);
    B5 = X1 + X2;
    Temperature = (B5 + 8) / pow(2, 4);
    
    return Temperature;
}

/*
 * The function calculates the true preassure value in Pa
 * @param:
 *          long UP - uncompensated preassure value
 *          long UT - uncompensated temperature value
 *          BMP180_CALIBRATION_DATA calibration - calibration data
 * @returns: true preassure value
 */
long calculateTruePres(long UP, long UT, BMP180_CALIBRATION_DATA calibration) {
    long Pressure; 
    long X1, X2, X3, B3, B5, B6; 
    unsigned long B4, B7;
    
    X1 = (UT - calibration.AC6) * calibration.AC5 / pow(2, 15);
    X2 = calibration.MC * pow(2, 11) / (X1 + calibration.MD);
    B5 = X1 + X2;
    B6 = B5 - 4000;
    X1 = (calibration.B2 * (B6 * B6 / pow(2, 12))) / pow(2, 11);
    X2 = calibration.AC2 * B6 / pow(2, 11);
    X3 = X1 + X2;
    B3 = (((calibration.AC1 * 4 + X3) << OSS) + 2) / 4;
    X1 = calibration.AC3 * B6 / pow(2, 13);
    X2 = (calibration.B1 * (B6 * B6 / pow(2, 12))) / pow(2, 16);
    X3 = ((X1 + X2) + 2) / pow(2, 2);
    B4 = (calibration.AC4 * (unsigned long)(X3 + 32768)) / pow(2, 15);
    B7 = ((unsigned long)UP - B3) * (50000 >> OSS);
    if(B7 < 0x80000000)
    {
        Pressure = (B7 *2) / B4;
    }
    else 
    {
        Pressure = (B7 / B4) *2;
    }
    X1 = (Pressure / pow(2, 8)) * (Pressure / pow(2, 8));
    X1 = (X1 * 3038) / pow(2, 16);
    X2 = (-7357 * Pressure) / pow(2, 16);
    Pressure = Pressure + (X1 + X2 + 3791) / pow(2, 4);
    
    return Pressure;
}

/*
 * The function gets the temperature value from BMP180
 * @param: 
 *          BMP180_CALIBRATION_DATA calibration - calibration data
 * @returns: temperature value from BMP180
 */
long BMP180GetTemperature(BMP180_CALIBRATION_DATA calibration){
    long temperature, ut; 
    
    ut = BMP180_Read_UT();
    temperature = calculateTrueTemp(ut,calibration);
    
    return temperature;
}

/*
 * The function gets the preassure value from BMP180
 * @param: 
 *          oss - oversampling setting bit (values from 0 to 3)
 *          BMP180_CALIBRATION_DATA calibration - calibration data
 * @returns: preassure value from BMP180
 */
long BMP180GetPreassure(BMP180_CALIBRATION_DATA calibration){
    long preassure,up,ut; 
    
    up = BMP180_Read_UP();
    ut = BMP180_Read_UT();
    preassure = calculateTruePres(up,ut,calibration);
    
    return preassure;
}
/* [] END OF FILE */
