#include "DHT22.h"

/* DHT22 status code definitions */ 
const unsigned char *DHT22_GetReturnCodeString(DHT22_ErrorCode code) {
  switch(code) {
    case DHT22_OK:                                      return "DHT22 is working fine";
    case DHT22_NoZeroAcknowledgement:                   return "DHT22 did not send 0 acknowledgemnt";
    case DHT22_NoOneAcknowledgemnt:                     return "DHT22 did not send 1 acknowledgemnt";
    case DHT22_LowDataExpectedDuringTransmission:       return "DHT22 did not pull to low, during transmission";
    case DHT22_HighDataExpectedDuringTransmission:      return "DHT22 did not pull to high, during transmission";
    default:                                            return "Unknown error";
  }
}

DHT22_ErrorCode DHT22_Read(uint16_t *temperatureCentigrade, uint16_t *humidityCentipercent) {
  int cntr;
  int loopBits;
  uint8_t buffer[5];
  int i;
  int data;

  /* initialise the data buffer */
  for(i=0;i<sizeof(buffer); i++) {
    buffer[i] = 0;
  }
  /* turn of interupt processing so serial data input is not interupted */
  CyGlobalIntDisable;

  /* send the start signal */
  DHT22_Data_Write(LOW);
  CyDelay(1);               //  wait for 1 ms
  DHT22_Data_Write(HIGH);

  /* check for acknowledge signal from DHT22 */
  /* if no start of acknowledgem is sent for 85 us return no zero acknoweldgment*/
  cntr = 17;
  while (DHT22_Data_Read()!=0) { // signal must be pulled low by the sensor 
    CyDelayUs(5);
    if (--cntr==0){
        return DHT22_NoZeroAcknowledgement;
    }
  }
  /* if acknowledgemnt signal is held higher than 90 us return no one akcnoldgment */
  cntr = 18;
  while(DHT22_Data_Read()==0) { // wait until signal goes up 
    CyDelayUs(5);
    if (--cntr==0) {
      return DHT22_NoOneAcknowledgemnt; 
    }
  }
  /* wait for end of acknowledgment */ 
  /* if no end of acknowledgement is sent return no zero acknoweldgment  */
  cntr = 18;
  while(DHT22_Data_Read()!=0) { // wait until signal goes down 
    CyDelayUs(5);
    if (--cntr==0) {
      return DHT22_NoZeroAcknowledgement; 
    }
  }
  /* Procces the 40 bits of data */
  i = 0;
  data = 0;
  loopBits = 40;
  do {
    cntr = 11; // wait max 55 us 
    /* if there is no change between bits resturn expected low data during transmission */
    while(DHT22_Data_Read()==0) {
      CyDelayUs(5);
      if (--cntr==0) {
        return DHT22_LowDataExpectedDuringTransmission;
      }
    }
    cntr = 15; // wait max 75 us 
    /* if there is no change between bits resturn expected high data during transmission */
    while(DHT22_Data_Read()!=0) {
      CyDelayUs(5);
      if (--cntr==0) {
        return DHT22_HighDataExpectedDuringTransmission;
      }
    }
    data <<= 1; // next data bit 
    if (cntr<10) { // data signal high > 30 us ==> data bit 1 
      data |= 1;
    }
    if ((loopBits&0x7)==1) { // next byte 
      buffer[i] = data;
      i++;
      data = 0;
    }
  } while(--loopBits!=0);
  /* re-enabling interrupts */
  CyGlobalIntEnable; 
 
  /* store data values for caller */

  *humidityCentipercent = (((int)buffer[0]<<8)+buffer[1])*10;
  *temperatureCentigrade = (((int)buffer[2]<<8)+buffer[3])*10;

  return DHT22_OK;
}
/* [] END OF FILE */
